/****************************************************************************
**
** Copyright (C) 2008-2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (developer.feedback@nokia.com)
**
** This file is part of the HbWidgets module of the UI Extensions for Mobile.
**
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser General Public
** License version 2.1 as published by the Free Software Foundation and
** appearing in the file LICENSE.LGPL included in the packaging of this file.
** Please review the following information to ensure the GNU Lesser General
** Public License version 2.1 requirements will be met:
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** If you have questions regarding the use of this file, please contact
** Nokia at developer.feedback@nokia.com.
**
****************************************************************************/

#include "hbdatepickerviewitem_p.h"
#include "hbtapgesture_p.h"

#include "hblistviewitem_p.h"
#include <QGestureEvent>
#include <QCoreApplication>
#include <hbwidgetfeedback.h>
class HbDatePickerViewItemPrivate:public HbListViewItemPrivate
{
public:
    HbDatePickerViewItemPrivate(HbDatePickerViewItem *prototype);
    void tapTriggered(QGestureEvent *event);

    Q_DECLARE_PUBLIC(HbDatePickerViewItem)
};
HbDatePickerViewItemPrivate::HbDatePickerViewItemPrivate(HbDatePickerViewItem *prototype):
        HbListViewItemPrivate(prototype)
{
}

void HbDatePickerViewItemPrivate::tapTriggered(QGestureEvent *event)
{
    Q_Q(HbDatePickerViewItem);

    HbTapGesture *gesture = static_cast<HbTapGesture *>(event->gesture(Qt::TapGesture));
    QPointF position = event->mapToGraphicsScene(gesture->hotSpot());
    position = q->mapFromScene(position);

    switch (gesture->state()) {
        case Qt::GestureStarted: {
            setPressed(true, true);
            emit q->pressed(position);
            break;
        }
        case Qt::GestureUpdated: {
            if (gesture->tapStyleHint() == HbTapGesture::TapAndHold
                && mSharedData->mItemView
                && mSharedData->mItemView->longPressEnabled()) {
                setPressed(false, true);

            }
            break;
        }
        case Qt::GestureFinished: {
            if (gesture->tapStyleHint() == HbTapGesture::Tap
                || (mSharedData->mItemView
                && !mSharedData->mItemView->longPressEnabled())) {
                setPressed(false, true);

                HbWidgetFeedback::triggered(q, Hb::InstantReleased, 0);
                HbWidgetFeedback::triggered(q, Hb::InstantClicked);
                QPointer<HbAbstractViewItem> item = q;
              //  emit item->activated(position);
                // this viewItem may be deleted in the signal handling, so guarded pointer is used to
                // to ensure that the item still exists when item is used
                if (item) {
                    q->emitReleased(position);

                }
            } else {
                HbWidgetFeedback::triggered(q, Hb::InstantReleased,0);
                q->emitReleased(position);
            }

            break;
        }
        case Qt::GestureCanceled: {
            // hides focus immediately
            setPressed(false, false);

            emit q->released(position);
            break;
        }
        default:
            break;
    }

    event->accept();

}
HbDatePickerViewItem::HbDatePickerViewItem(QGraphicsItem* parent)
    :HbListViewItem(*(new HbDatePickerViewItemPrivate(this)),parent)
{
    HB_SDD(HbAbstractViewItem);
    sd->mItemType = QString("datepickerviewitem");
    setFocusPolicy(Qt::NoFocus);
}

HbDatePickerViewItem::HbDatePickerViewItem(const HbDatePickerViewItem& other)
    :HbListViewItem(*(new HbDatePickerViewItemPrivate(*other.d_func())),0)
{
    Q_D(HbDatePickerViewItem);
    d->q_ptr = this;
    d->init();
}

HbAbstractViewItem *HbDatePickerViewItem::createItem()
{
    HbDatePickerViewItem* item = new HbDatePickerViewItem(*this);
    item->setFocusPolicy(Qt::NoFocus);
    connect(item,SIGNAL(released(QPointF)),item->itemView(),SLOT(_q_itemSelected(QPointF)));
    return item;
}

void HbDatePickerViewItem::updateChildItems()
{
    Q_D(HbDatePickerViewItem);
    if (d->mIndex.data(Qt::DisplayRole).isNull())
         return;
    HbListViewItem::updateChildItems();
    if(d->mSelectionItem){
        d->mSelectionItem->hide();
    }
}

void HbDatePickerViewItem::gestureEvent(QGestureEvent *event)
{
    HbTapGesture *gesture = static_cast<HbTapGesture *>(event->gesture(Qt::TapGesture));
    if (gesture) {
        Q_D(HbDatePickerViewItem);
        if(gesture->state() == Qt::GestureCanceled){
            d->setPressed(false, false);
            return;
        }
        if(itemView()->isScrolling()){
            event->ignore();
            event->ignore(Qt::TapGesture);
            return;
        }
        d->tapTriggered(event);
    } else {
        HbWidget::gestureEvent(event);

    }
}

QSizeF HbDatePickerViewItem::sizeHint(Qt::SizeHint which, const QSizeF &constraint)
{
    HB_SDD(const HbAbstractViewItem);
    static qreal height=0;

    QSizeF sh=HbListViewItem::sizeHint(which,constraint);
    if(which == Qt::PreferredSize && sh.height()<=0) {
        //TODO:remove this check once refresh issue in DTP is solved on removeRows/insertRows
        if(height<=0) {
            //Let's create a temporary item and take the height for the size hint
            HbAbstractViewItem *tempitem = sd->mPrototype->createItem();
            if(sd->mItemView) {
                QAbstractItemModel *model = sd->mItemView->model();
                if(model) {
                    int rowCount=model->rowCount();
                    QModelIndex ind=model->index(rowCount,0);
                    if(ind.isValid()) {
                        tempitem->setModelIndex(ind);
                    }
                }
            }


            //call polish
            QEvent polishEvent(QEvent::Polish);
            QCoreApplication::sendEvent(const_cast<HbAbstractViewItem *>(tempitem), &polishEvent);
            height=tempitem->effectiveSizeHint(which,constraint).height();
            delete tempitem;
        }
        return QSizeF(sh.width(),height);
    }
    return sh;
}

void HbDatePickerViewItem::emitReleased(const QPointF& point)
{
     emit HbAbstractViewItem::released(point);
}

/*!
    \reimp
*/
int HbDatePickerViewItem::type() const
{
    return Type;
}
